;---------------------------------------------------------------------
; file          : stdio.inc
; created       : 2001/03/11
; assembler     : nasm 0.98
; description   : standerd input/output version 0.4
;
;Copyright (C) 2000,2001 Jun Mizutani <mizutani.jun@nifty.ne.jp>
;---------------------------------------------------------------------

%ifndef __STDIO_INC
%define __STDIO_INC

%ifndef __SYSCALL_INC
%include "syscall.inc"
%endif

;------------------------------------
; exit with 0
Exit:
               mov    eax, 1         ; sys_exit
               mov    ebx, 0         ; exit with code 0
               int    0x80

;------------------------------------
; exit with ebx
ExitN:
               mov    ebx, eax       ; exit with code ebx
               mov    eax, 1         ; sys_exit
               int    0x80


;------------------------------------
; print string to stdout
; eax : top address
; edx : no of put char
OutString:
               pusha
               mov    ecx, eax
               mov    eax, SYS_write
               mov    ebx, 1         ; to stdout
               int    0x80
               popa
               ret

;------------------------------------
; print pascal string to stdout
; ebx : top address
OutPString
                push   eax
                push   ebx
                push   edx
                xor    edx, edx
                mov    dl, [ebx]
                inc    ebx
                mov    eax, ebx
                call   OutString
                pop    edx
                pop    ebx
                pop    eax
                ret

;------------------------------------
; get length of asciiz string
; eax : top address
; eax : return length
StrLen:
               push   ecx
               push   edi
               mov    edi, eax
               push   eax
               xor    eax, eax
               mov    ecx, 0xFFFF  ; no more than 65k chars.
         repne scasb
               pop    ecx
               sub    edi, ecx
               mov    eax, edi
               pop    edi
               pop    ecx
               ret

;------------------------------------
; print asciiz string
; eax : pointer to string
OutAsciiZ:
               push   edx
               push   eax
               call   StrLen
               mov    edx, eax
               pop    eax
               call   OutString
               pop    edx
               ret

;------------------------------------
; print 1 character to stdout
; eax : put char
OutChar:
               pusha
               push   eax            ; work buffer on stack
               mov    eax, SYS_write
               mov    ebx, 1         ; to stdout
               mov    edx, 1         ; 1 char
               mov    ecx, esp
               int    0x80
               pop    eax
               popa
               ret

;------------------------------------
; print 4 characters in eax to stdout
;   destroyed : eax
OutChar4:
               push   ecx
               mov    ecx, 0x0408
.loop
               and    al, 0x7F       ; 7bit only
               cmp    al, 0x7F
               jz     .dot
               cmp    al, 0x20
               jae    .output
.dot           mov    al, '.'
.output        call   OutChar
               shr    eax, cl
               dec    ch
               jnz    .loop
               pop    ecx
               ret

;------------------------------------
; new line
;   all registers are preserved.
NewLine:
               push   eax
               mov    al, 0AH
               call   OutChar
               pop    eax
               ret

;------------------------------------
; Backspace
;   destroyed : al
BackSpace:
               mov    al, 0x08
               call   OutChar
               mov    al, ' '
               call   OutChar
               mov    al, 0x08
               call   OutChar
               ret

;------------------------------------
; print 2 digit hex number (lower 8 bit of eax)
;   eax : number
;   destroyed : edx
PrintHex2:
               mov    edx, 2
               jmp    short PrintHex

;------------------------------------
; print 4 digit hex number (lower 16 bit of eax)
;   eax : number
;   destroyed : edx
;
PrintHex4:
               mov    edx, 4
               jmp    short PrintHex

;------------------------------------
; print 8 digit hex number (eax)
;   eax : number
;   destroyed : edx
;
PrintHex8:
               mov    edx, 8

;------------------------------------
; print hex number
;   eax : number     edx : digit
;
PrintHex:
               push   eax
               push   ecx
               push   ebx
               mov    ecx, edx
  .loop1:      mov    bl, al
               and    bl, 0x0F
               shr    eax, 4
               or     bl, 0x30
               cmp    bl, 0x3A
               jb     .skip
               add    bl, 0x41 - 0x3A   ; A-F
  .skip:
               push   ebx
               loop   .loop1
               mov    ecx, edx
  .loop2:      pop    eax
               call   OutChar
               loop   .loop2
               pop    ebx
               pop    ecx
               pop    eax
               ret

;------------------------------------
; Output Unsigned Number to stdout
; eax : number
PrintLeftU:
                pusha
                xor     ecx, ecx            ; JE^
                xor     edi, edi            ; 
                jmp     short PrintLeft.positive

;------------------------------------
; Output Number to stdout
; eax : number
PrintLeft:
                pusha
                xor     ecx, ecx            ; JE^
                xor     edi, edi            ; 
                test    eax, eax
                jns     .positive
                inc     edi                 ; ɐݒ
                neg     eax
    .positive:  mov     ebx, 10
    .PL1:       xor     edx, edx            ; ʌ 0 
                div     ebx                 ; 10 ŏZ
                push    edx                 ; ](ʌ)PUSH
                inc     ecx                 ; XV
                test    eax, eax            ; I?
                jnz     .PL1
    .PL2:       test    edi, edi
                je      .pos
                mov     al, '-'             ; R[hɕύX
                call    OutChar             ; o
    .pos:       pop     eax                 ; ʌ POP
                add     al, '0'             ; R[hɕύX
                call    OutChar             ; o
                loop    .pos
                popa
                ret

;------------------------------------
; Output Number to stdout
; ecx:column
; eax:number
PrintRight0:
                pusha
                mov     ebp, '0'
                jmp     short PrintRightU.pr0

;------------------------------------
; Output Unsigned Number to stdout
; ecx:column
; eax:number
PrintRightU:
                pusha
                mov     ebp, ' '
    .pr0:       mov     esi, ecx            ; \ esi 
                xor     ecx, ecx            ; JE^
                xor     edi, edi            ; 
                jmp     short PrintRight.positive

;------------------------------------
; Output Number to stdout
; ecx:column
; eax:number
PrintRight:
                pusha
                mov     ebp, ' '
    .pr0:       mov     esi, ecx            ; \ esi 
                xor     ecx, ecx            ; JE^
                xor     edi, edi            ; 
                test    eax, eax
                jns     .positive
                dec     esi
                inc     edi                 ; ݒ
                neg     eax
    .positive:
                mov     ebx, 10
    .pr1:       xor     edx, edx            ; ʌ 0 
                div     ebx                 ; 10 ŏZ
                push    edx                 ; ](ʌ)PUSH
                inc     ecx                 ; XV
                test    eax, eax            ; I?
                jnz     .pr1
                sub     esi, ecx            ; esi ɃXy[X
                jbe     .done               ; \𒴂
                xchg    esi, ecx            ; ecx ɃXy[X
    .space:     mov     eax, ebp            ; Xy[X 0
                call    OutChar             ; Xy[Xo
                loop    .space
                xchg    esi, ecx            ; ecx ɕ\
    .done:
                jmp     short PrintLeft.PL2


;------------------------------------
; input 1 character from stdin
; eax : get char
InChar:
               push   ebx
               push   ecx
               push   edx
               push   eax            ; work buffer on stack
               mov    eax, SYS_read
               mov    ebx, 0         ; from stdin
               mov    ecx, esp       ; into Input Buffer
               mov    edx, 1         ; 1 char
               int    0x80           ; call kernel
               pop    eax
               pop    edx
               pop    ecx
               pop    ebx
               ret

;------------------------------------
; Input Line
; eax : BufferSize
; ebx : Buffer Address
; return       eax : no. of char
;
InputLine0:
               push   edi
               push   ecx
               push   edx
               mov    edx, eax
               mov    edi, ebx       ; Input Buffer
               xor    ecx, ecx
   .in_char:
               call   InChar
               cmp    al, 0x08       ; BS ?
               jnz    .in_char2
               test   ecx, ecx
               jz     .in_char2
               call   BackSpace      ; backspace
               dec    ecx
               jmp    short .in_char
   .in_char2:
               cmp    al, 0x0A       ; enter ?
               jz     .in_exit

   .in_printable:
               call   OutChar
               mov    [edi + ecx], al
               inc    ecx
               cmp    ecx, edx       ;
               jae    .in_toolong
               jmp    short .in_char
   .in_toolong:
               dec    ecx
               call   BackSpace
               jmp    short .in_char

   .in_exit:
               mov    dword [edi + ecx], 0
               inc    ecx
               call   NewLine
               mov    eax, ecx
               pop    ecx
               pop    edi
               ret

%endif
